﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Interactivity;
using System.Windows;
using System.Reflection;
using System.Reflection.Emit;

namespace MvvmLight2
{
    public class RoutedEventTrigger<T> : System.Windows.Interactivity.EventTriggerBase<FrameworkElement> where T:RoutedEventArgs
    {
        /// <summary>
        /// Gets or sets the owner of the event. This property is used to resolve the correct routed event.
        /// </summary>
        public Type EventOwnerType
        {
            get { return (Type)GetValue(EventOwnerTypeProperty); }
            set { SetValue(EventOwnerTypeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for EventOwnerType.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty EventOwnerTypeProperty =
            DependencyProperty.Register("EventOwnerType", typeof(Type), typeof(RoutedEventTrigger<T>), new PropertyMetadata(null, OnEventOwnerTypeChanged));

        private static void OnEventOwnerTypeChanged(DependencyObject o, DependencyPropertyChangedEventArgs args)
        {
            var owner = o as RoutedEventTrigger<T>;

            if (owner != null && owner.CanSubscribeToEvent())
            {
                owner.SubscribeToEvent();
            }
        }

        protected override string GetEventName()
        {
            return this.EventName;
        }

        /// <summary>
        /// Gets or sets the Routed Event name.
        /// </summary>
        public string EventName
        {
            get { return (string)GetValue(EventNameProperty); }
            set { SetValue(EventNameProperty, value); }
        }

        // Using a DependencyProperty as the backing store for EventPropertyName.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty EventNameProperty =
            DependencyProperty.Register("EventName", typeof(string), typeof(RoutedEventTrigger<T>), new UIPropertyMetadata(null, OnEventNameChanged));

        private static void OnEventNameChanged(DependencyObject o, DependencyPropertyChangedEventArgs args)
        {
            var owner = o as RoutedEventTrigger<T>;

            if (owner != null && owner.CanSubscribeToEvent())
            {
                owner.SubscribeToEvent();
            }
        }

        protected override void OnAttached()
        {
            base.OnAttached();

            if (this.CanSubscribeToEvent())
            {
                SubscribeToEvent();
            }
        }

        private bool CanSubscribeToEvent()
        {
            return EventName != null && EventOwnerType != null;
        }

        protected virtual void SubscribeToEvent()
        {
            var routedEvent = EventManager.GetRoutedEventsForOwner(this.EventOwnerType).Where(c => c.Name == this.EventName).First();

            if (routedEvent != null)
            {
                Type handlerType = routedEvent.HandlerType;
                var del = Delegate.CreateDelegate(handlerType, this, "OnEventInvoked");

                if (this.AssociatedObject is UIElement)
                {
                    ((UIElement)this.AssociatedObject).AddHandler(routedEvent, del);
                }
            }
        }

        protected virtual  void OnEventInvoked(object o, T args)
        {
            this.InvokeActions(args);
        }


    }
}
